import pycodestyle
from lib.InternalTestBase import InternalTestBase


class TestPep8(InternalTestBase):
    """ Test that we conform to PEP8. """

    def test_pep8(self):
        pep8style = pycodestyle.StyleGuide(quiet=False)
        result = pep8style.check_files(self.list_project_files())
        assert result.total_errors == 0, "Found pep8 problems"



import re
import fnmatch
from os import path, walk


class InternalTestBase(object):
    ROOT_DIR = None
    TEST_DIR = None
    LIB_DIR = None

    def list_project_files(self, exclude=None):
        """ List python project files """
        if exclude is None:
            exclude = ["env"]

        # compile asterisks to regular expressions
        exclude_regex = []
        for excluded in [path.join(self.ROOT_DIR, excluded) for excluded in
                         exclude]:
            escaped = re.escape(excluded)
            escaped = escaped.replace(
                "\\*\\*", ".*")  # pylint: disable=W1401
            escaped = escaped.replace(
                "\\*", "[^\\/]*")  # pylint: disable=W1401
            exclude_regex.append(re.compile(escaped))

        result = []
        for root, _, files in walk(self.ROOT_DIR):
            for file_ in fnmatch.filter(files, "*.py"):
                abs_path = path.join(root, file_)
                include = True
                for excluded in exclude_regex:
                    if excluded.match(abs_path):
                        include = False
                        break
                if include:
                    result.append(abs_path)

        return result



import re
import os
import subprocess
from lib.InternalTestBase import InternalTestBase


class TestVersionConsistent(InternalTestBase):
    """ Test setup.py version doesn't fall behind git tag. """

    def test_version_consistent(self):
        v1 = subprocess.check_output(
            'git describe --tags --abbrev=0'.split()).strip().decode("utf-8")
        with open(os.path.join(self.ROOT_DIR, "setup.py")) as f:
            v2 = re.search("version='([0-9.]+)',", f.read()).groups()[0]
        assert list(map(int, v1.split("."))) <= list(map(int, v2.split(".")))



import re
from lib.InternalTestBase import InternalTestBase


class TestIncorrectBoolConditional(InternalTestBase):
    """
        Test asserts don't check 'in (True, False)' (false positives)
        Use `isinstance(val, bool)` instead
    """

    def test_incorrect_bool_conditional(self):
        assert_regex_compiled = re.compile(
            r'.*?\sin\s[\[(][^)\]]*?(True|False)[^)\]]*?[\])]'
        )
        matches = []
        for file_ in self.list_project_files():
            with open(file_, 'r') as f:
                for l in f.readlines():
                    match = assert_regex_compiled.match(l)
                    if match is not None:  # pragma: no cover
                        matches.append('%s: %s' % (
                            f.name.split('/')[-1], match.string.strip()))
        assert len(matches) == 0, 'Invalid assert(s):\n' + '\n'.join(matches)



import os
import re
import multiprocessing
from multiprocessing.pool import ThreadPool
import time
from pylint import epylint
from lib.InternalTestBase import InternalTestBase


class TestPylint(InternalTestBase):
    """ Test that we conform to Pylint. """

    suppress_for_test_files = [
        "E1101",  # no-member (Model creation)
        "W0221",  # arguments-differ (class setup inheritance)
        "W0212"  # protected-access  (e.g. private methods)
    ]

    def test_pylint(self):
        pool = ThreadPool(multiprocessing.cpu_count())
        processes = [pool.apply_async(
            lambda f: epylint.py_run('%s --rcfile=%s' % (
                f,
                os.path.join(self.ROOT_DIR, ".pylintrc")
            ), return_std=True),
            [file_]
        ) for file_ in self.list_project_files()]

        while not all(p.ready() for p in processes):
            time.sleep(0.2)

        logs = []
        for process in processes:
            stdout, stderr = map(lambda e: e.getvalue(), process.get())
            if "Your code has been rated at 10.00/10" not in stdout:
                for log in stdout.split("\n"):
                    if re.match(r"^\*{13} Module [a-zA-Z0-9._]+$", log):
                        continue
                    if re.match(r"^\s[-]+$", log):
                        continue
                    if re.match(
                        r"^\sYour code has been rated at \d+\.\d+/10 "
                        r"\(previous run: \d+\.\d+/10, [+\-]\d+\.\d+\)$",
                        log
                    ):
                        continue
                    if re.match(r"^\s*$", log):
                        continue
                    logs.append(log.strip())
            if not re.match(
                r"^Using config file [/A-Za-z0-9\-_]+?\.pylintrc\n$",
                stderr
            ):
                logs.append(stderr)
        pool.close()

        result = []
        if len(logs) > 0:
            for log in logs:
                m = re.match(
                    r"^([/a-zA-Z0-9_.\-]+):(\d+): (error|warning) "
                    r"\(([A-Z]\d{4}), ([a-z\-]+), ",
                    log
                )
                if m:
                    groups = m.groups()
                    prefix = os.path.commonprefix([self.ROOT_DIR, groups[0]])
                    relpath = os.path.relpath(groups[0], prefix)
                    if (
                        relpath.startswith("tests/") and
                        groups[3] in self.suppress_for_test_files
                    ):
                        continue
                result.append(log)

        if len(result) > 0:
            for log in result:
                print(log)
            print("PyLint Problems: %s" % len(result))
        assert len(result) == 0



from lib.InternalTestBase import InternalTestBase


class TestLineEndings(InternalTestBase):
    """ Test that lines do not end with backslash - use parenthesis instead """

    def test_line_endings(self):
        for file_ in self.list_project_files():
            with open(file_, 'r') as f:
                for l in f.readlines():
                    assert not l.rstrip().endswith("\\"), file_ + " @ " + l
